#include <gtk/gtk.h>
#include <gst/gst.h>
#include <glib.h>

#include <gdk/gdkx.h>
#include <gst/interfaces/xoverlay.h>

/* Bad programming I know, but for quick testing 
 * global variable videodisplay for storing the DrawingArea
 * widget */

GtkWidget *videodisplay;

/* define the ui file */
#define UI_FILE "test.ui"

/* Function to Create the Gtk window from the .ui file */
GtkWidget*
create_window (void)
{
	GtkWidget *window;
	GtkBuilder *builder;
	GError* error = NULL;

	builder = gtk_builder_new ();
	if (!gtk_builder_add_from_file (builder, UI_FILE, &error))
	{
		g_warning ("Couldn't load builder file: %s", error->message);
		g_error_free (error);
	}

	/* This is important */
	gtk_builder_connect_signals (builder, NULL);
	window = GTK_WIDGET (gtk_builder_get_object (builder, "window"));

	/* Get the DrawingArea "vid1" into the global variable videodisplay */
	videodisplay = GTK_WIDGET (gtk_builder_get_object (builder, "vid1"));

	/* Uncomment and comment above line to test overlaying main window */
	/* videodisplay = window; */

	g_object_unref (builder);
	
	return window;
}

/* The Gst Bus function to overlay the xvimagesink when the "prepare-xwindow-id" message is caught */
GstBusSyncReply
setup_window (GstBus * bus, GstMessage * message, GstPipeline * pipeline)
{
	GdkWindow* videowindow;

	// ignore anything but 'prepare-xwindow-id' element messages
	if (GST_MESSAGE_TYPE (message) != GST_MESSAGE_ELEMENT)
	return GST_BUS_PASS;
 
	if (!gst_structure_has_name (message->structure, "prepare-xwindow-id"))
	return GST_BUS_PASS;

	/* Get GdkWindow from videodisplay (drawingarea vid1) variable */
	videowindow = gtk_widget_get_window(GTK_WIDGET (videodisplay));
	
	gst_x_overlay_set_xwindow_id (GST_X_OVERLAY (GST_MESSAGE_SRC (message)), GDK_WINDOW_XID(videowindow));

	gst_message_unref (message);
  
	return GST_BUS_DROP;
}

int
main (int argc, char *argv[])
{
 	GtkWidget *window;
	GstElement *source, *sink, *pipeline;
	GstBus *bus;
	GMainLoop *loop;

	gtk_init (&argc, &argv);
	gst_init (NULL, NULL);

	loop = g_main_loop_new (NULL, FALSE);

	/* Create gstreamer elements */
	pipeline = gst_pipeline_new ("video-test");
	source   = gst_element_factory_make ("videotestsrc", "video-source");
	sink     = gst_element_factory_make ("xvimagesink", "video-output");

	if (!pipeline || !source || !sink) {
	g_printerr ("One element could not be created. Exiting.\n");
	return -1;
	}

	bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
	gst_bus_set_sync_handler (bus, (GstBusSyncHandler) setup_window, pipeline);
	gst_object_unref (bus);

	/* add all elements into the pipeline */
	gst_bin_add_many (GST_BIN (pipeline),
		            source, sink, NULL);

	/* Lock the aspect ratio */
	g_object_set(G_OBJECT(sink), "force-aspect-ratio", TRUE, NULL);

	/* link the elements together */
	gst_element_link_many (source, sink, NULL);

	/* Create the GTK window */
	window = create_window ();
	gtk_widget_show_all (window);

	/* Set the pipeline to "playing" state*/
	g_print ("Now playing\n");
	gst_element_set_state (pipeline, GST_STATE_PLAYING);

 	gtk_main ();

	/* g_main_loop_run (loop); */

	/* Out of the main loop, clean up nicely */
	g_print ("Returned, stopping playback\n");
	gst_element_set_state (pipeline, GST_STATE_NULL);

	g_print ("Deleting pipeline\n");
	gst_object_unref (GST_OBJECT (pipeline));
	
	return 0;
}

